@@ -25,6 +25,24 @@
 
 #include "nm-setting-ppp.h"
 
+/**
+ * SECTION:nm-setting-ppp
+ * @short_description: Describes connection properties for devices/networks
+ * that require PPP to deliver IP capability
+ * @include: nm-setting-ppp.h
+ *
+ * The #NMSettingPPP object is a #NMSetting subclass that describes properties
+ * necessary for connection to networks that require PPP transport, like PPPoE
+ * cable and DSL modems and some mobile broadband devices.
+ **/
+
+/**
+ * nm_setting_ppp_error_quark:
+ *
+ * Registers an error quark for #NMSettingPPP if necessary.
+ *
+ * Returns: the error quark used for #NMSettingPPP errors.
+ **/
 GQuark
 nm_setting_ppp_error_quark (void)
 {
@@ -110,12 +128,25 @@ enum {
 	LAST_PROP
 };
 
+/**
+ * nm_setting_ppp_new:
+ *
+ * Creates a new #NMSettingPPP object with default values.
+ *
+ * Returns: (transfer full): the new empty #NMSettingPPP object
+ **/
 NMSetting *
 nm_setting_ppp_new (void)
 {
 	return (NMSetting *) g_object_new (NM_TYPE_SETTING_PPP, NULL);
 }
 
+/**
+ * nm_setting_wired_get_noauth:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:noauth property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_noauth (NMSettingPPP *setting)
 {
@@ -124,6 +155,12 @@ nm_setting_ppp_get_noauth (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->noauth;
 }
 
+/**
+ * nm_setting_wired_get_refuse_eap:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:refuse-eap property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_refuse_eap (NMSettingPPP *setting)
 {
@@ -132,6 +169,12 @@ nm_setting_ppp_get_refuse_eap (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->refuse_eap;
 }
 
+/**
+ * nm_setting_wired_get_refuse_pap:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:refuse-pap property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_refuse_pap (NMSettingPPP *setting)
 {
@@ -140,6 +183,12 @@ nm_setting_ppp_get_refuse_pap (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->refuse_pap;
 }
 
+/**
+ * nm_setting_wired_get_refuse_chap:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:refuse-chap property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_refuse_chap (NMSettingPPP *setting)
 {
@@ -148,6 +197,12 @@ nm_setting_ppp_get_refuse_chap (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->refuse_chap;
 }
 
+/**
+ * nm_setting_wired_get_refuse_mschap:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:refuse-mschap property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_refuse_mschap (NMSettingPPP *setting)
 {
@@ -156,6 +211,12 @@ nm_setting_ppp_get_refuse_mschap (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->refuse_mschap;
 }
 
+/**
+ * nm_setting_wired_get_refuse_mschapv2:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:refuse-mschapv2 property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_refuse_mschapv2 (NMSettingPPP *setting)
 {
@@ -164,6 +225,12 @@ nm_setting_ppp_get_refuse_mschapv2 (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->refuse_mschapv2;
 }
 
+/**
+ * nm_setting_wired_get_nobsdcomp:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:nobsdcomp property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_nobsdcomp (NMSettingPPP *setting)
 {
@@ -172,6 +239,12 @@ nm_setting_ppp_get_nobsdcomp (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->nobsdcomp;
 }
 
+/**
+ * nm_setting_wired_get_nodeflate:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:nodeflate property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_nodeflate (NMSettingPPP *setting)
 {
@@ -180,6 +253,12 @@ nm_setting_ppp_get_nodeflate (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->nodeflate;
 }
 
+/**
+ * nm_setting_wired_get_no_vj_comp:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:no-vj-comp property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_no_vj_comp (NMSettingPPP *setting)
 {
@@ -188,6 +267,12 @@ nm_setting_ppp_get_no_vj_comp (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->no_vj_comp;
 }
 
+/**
+ * nm_setting_wired_get_require_mppe:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:require-mppe property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_require_mppe (NMSettingPPP *setting)
 {
@@ -196,6 +281,12 @@ nm_setting_ppp_get_require_mppe (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->require_mppe;
 }
 
+/**
+ * nm_setting_wired_get_require_mppe_128:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:require-mppe-128 property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_require_mppe_128 (NMSettingPPP *setting)
 {
@@ -204,6 +295,12 @@ nm_setting_ppp_get_require_mppe_128 (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->require_mppe_128;
 }
 
+/**
+ * nm_setting_wired_get_mppe_stateful:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:mppe-stateful property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_mppe_stateful (NMSettingPPP *setting)
 {
@@ -212,6 +309,12 @@ nm_setting_ppp_get_mppe_stateful (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->mppe_stateful;
 }
 
+/**
+ * nm_setting_wired_get_crtscts:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:crtscts property of the setting
+ **/
 gboolean
 nm_setting_ppp_get_crtscts (NMSettingPPP *setting)
 {
@@ -220,6 +323,12 @@ nm_setting_ppp_get_crtscts (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->crtscts;
 }
 
+/**
+ * nm_setting_wired_get_baud:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:baud property of the setting
+ **/
 guint32
 nm_setting_ppp_get_baud (NMSettingPPP *setting)
 {
@@ -228,6 +337,12 @@ nm_setting_ppp_get_baud (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->baud;
 }
 
+/**
+ * nm_setting_wired_get_mru:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:mru property of the setting
+ **/
 guint32
 nm_setting_ppp_get_mru (NMSettingPPP *setting)
 {
@@ -236,6 +351,12 @@ nm_setting_ppp_get_mru (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->mru;
 }
 
+/**
+ * nm_setting_wired_get_mtu:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:mtu property of the setting
+ **/
 guint32
 nm_setting_ppp_get_mtu (NMSettingPPP *setting)
 {
@@ -244,6 +365,12 @@ nm_setting_ppp_get_mtu (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->mtu;
 }
 
+/**
+ * nm_setting_wired_get_lcp_echo_failure:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:lcp-echo-failure property of the setting
+ **/
 guint32
 nm_setting_ppp_get_lcp_echo_failure (NMSettingPPP *setting)
 {
@@ -252,6 +379,12 @@ nm_setting_ppp_get_lcp_echo_failure (NMSettingPPP *setting)
 	return NM_SETTING_PPP_GET_PRIVATE (setting)->lcp_echo_failure;
 }
 
+/**
+ * nm_setting_wired_get_lcp_echo_interval:
+ * @setting: the #NMSettingPPP
+ *
+ * Returns: the #NMSettingPPP:lcp-echo-interval property of the setting
+ **/
 guint32
 nm_setting_ppp_get_lcp_echo_interval (NMSettingPPP *setting)
 {
@@ -263,7 +396,30 @@ nm_setting_ppp_get_lcp_echo_interval (NMSettingPPP *setting)
 static gboolean
 verify (NMSetting *setting, GSList *all_settings, GError **error)
 {
+	NMSettingPPPPrivate *priv = NM_SETTING_PPP_GET_PRIVATE (setting);
+
 	/* FIXME: Do we even want this or can we just let pppd evaluate the options? */
+	if (priv->mru > 0) {
+		if (priv->mru < 128 || priv->mru > 16384) {
+			g_set_error (error,
+			             NM_SETTING_PPP_ERROR,
+			             NM_SETTING_PPP_ERROR_INVALID_PROPERTY,
+			             NM_SETTING_PPP_MRU);
+			return FALSE;
+		}
+	}
+
+	if (priv->lcp_echo_failure > 0) {
+		/* lcp_echo_interval must also be non-zero */
+		if (priv->lcp_echo_interval == 0) {
+			g_set_error (error,
+			             NM_SETTING_PPP_ERROR,
+			             NM_SETTING_PPP_ERROR_INVALID_PROPERTY,
+			             NM_SETTING_PPP_LCP_ECHO_FAILURE);
+			return FALSE;
+		}
+	}
+
 	return TRUE;
 }
 
@@ -421,147 +577,287 @@ nm_setting_ppp_class_init (NMSettingPPPClass *setting_class)
 	parent_class->verify       = verify;
 
 	/* Properties */
+	/**
+	 * NMSettingPPP:noauth:
+	 *
+	 * If TRUE, do not require the other side (usually the PPP server) to
+	 * authenticate itself to the client.  If FALSE, require authentication from
+	 * the remote side.  In almost all cases, this should be TRUE.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_NOAUTH,
 		 g_param_spec_boolean (NM_SETTING_PPP_NOAUTH,
 						   "NoAuth",
-						   "NoAuth",
+						   "If TRUE, do not require the other side (usually the "
+						   "PPP server) to authenticate itself to the client.  "
+						   "If FALSE, require authentication from the remote "
+						   "side.  In almost all cases, this should be TRUE.",
 						   TRUE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:refuse-eap:
+	 *
+	 * If TRUE, the EAP authentication method will not be used.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_REFUSE_EAP,
 		 g_param_spec_boolean (NM_SETTING_PPP_REFUSE_EAP,
 						   "Refuse EAP",
-						   "Refuse EAP",
+						   "If TRUE, the EAP authentication method will not be used.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:refuse-pap:
+	 *
+	 * If TRUE, the PAP authentication method will not be used.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_REFUSE_PAP,
 		 g_param_spec_boolean (NM_SETTING_PPP_REFUSE_PAP,
 						   "Refuse PAP",
-						   "Refuse PAP",
+						   "If TRUE, the PAP authentication method will not be used.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:refuse-chap:
+	 *
+	 * If TRUE, the CHAP authentication method will not be used.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_REFUSE_CHAP,
 		 g_param_spec_boolean (NM_SETTING_PPP_REFUSE_CHAP,
 						   "Refuse CHAP",
-						   "Refuse CHAP",
+						   "If TRUE, the CHAP authentication method will not be used.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:refuse-mschap:
+	 *
+	 * If TRUE, the MSCHAP authentication method will not be used.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_REFUSE_MSCHAP,
 		 g_param_spec_boolean (NM_SETTING_PPP_REFUSE_MSCHAP,
 						   "Refuse MSCHAP",
-						   "Refuse MSCHAP",
+						   "If TRUE, the MSCHAP authentication method will not be used.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:refuse-mschapv2:
+	 *
+	 * If TRUE, the MSCHAPv2 authentication method will not be used.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_REFUSE_MSCHAPV2,
 		 g_param_spec_boolean (NM_SETTING_PPP_REFUSE_MSCHAPV2,
 						   "Refuse MSCHAPv2",
-						   "Refuse MSCHAPv2",
+						   "If TRUE, the MSCHAPv2 authentication method will not be used.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:nobsdcomp:
+	 *
+	 * If TRUE, BSD compression will not be requested.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_NOBSDCOMP,
 		 g_param_spec_boolean (NM_SETTING_PPP_NOBSDCOMP,
 						   "No BSD compression",
-						   "No BSD compression",
+						   "If TRUE, BSD compression will not be requested.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingPPP:nodeflate:
+	 *
+	 * If TRUE, 'deflate' compression will not be requested.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_NODEFLATE,
 		 g_param_spec_boolean (NM_SETTING_PPP_NODEFLATE,
 						   "No deflate",
-						   "No deflate",
+						   "If TRUE, 'deflate' compression will not be requested.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingPPP:no-vj-comp:
+	 *
+	 * If TRUE, Van Jacobsen TCP header compression will not be requested.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_NO_VJ_COMP,
 		 g_param_spec_boolean (NM_SETTING_PPP_NO_VJ_COMP,
 						   "No VJ compression",
-						   "No VJ compression",
+						   "If TRUE, Van Jacobsen TCP header compression will not be requested.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingPPP:require-mppe:
+	 *
+	 * If TRUE, MPPE (Microsoft Point-to-Point Encrpytion) will be required for
+	 * the PPP session.  If either 64-bit or 128-bit MPPE is not available the
+	 * session will fail.  Note that MPPE is not used on mobile broadband
+	 * connections.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_REQUIRE_MPPE,
 		 g_param_spec_boolean (NM_SETTING_PPP_REQUIRE_MPPE,
 						   "Require MPPE",
-						   "Require MPPE",
+						   "If TRUE, MPPE (Microsoft Point-to-Point Encrpytion) "
+						   "will be required for the PPP session.  If either "
+						   "64-bit or 128-bit MPPE is not available the session "
+						   "will fail.  Note that MPPE is not used on mobile "
+						   "broadband connections.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:require-mppe-128:
+	 *
+	 * If TRUE, 128-bit MPPE (Microsoft Point-to-Point Encrpytion) will be
+	 * required for the PPP session, and the 'require-mppe' property must also
+	 * be set to TRUE.  If 128-bit MPPE is not available the session will fail.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_REQUIRE_MPPE_128,
 		 g_param_spec_boolean (NM_SETTING_PPP_REQUIRE_MPPE_128,
 						   "Require MPPE 128",
-						   "Require MPPE 128",
+						   "If TRUE, 128-bit MPPE (Microsoft Point-to-Point "
+						   "Encrpytion) will be required for the PPP session, "
+						   "and the '" NM_SETTING_PPP_REQUIRE_MPPE "' property "
+						   "must also be set to TRUE.  If 128-bit MPPE is not "
+						   "available the session will fail.",
 						   FALSE,
 						   G_PARAM_READWRITE  | G_PARAM_CONSTRUCT| NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:mppe-stateful:
+	 *
+	 * If TRUE, stateful MPPE is used.  See pppd documentation for more
+	 * information on stateful MPPE.",
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_MPPE_STATEFUL,
 		 g_param_spec_boolean (NM_SETTING_PPP_MPPE_STATEFUL,
 						   "MPPE stateful",
-						   "MPPE stateful",
+						   "If TRUE, stateful MPPE is used.  See pppd documentation "
+						   "for more information on stateful MPPE.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:crtscts:
+	 *
+	 * If TRUE, specify that pppd should set the serial port to use hardware
+	 * flow control with RTS and CTS signals.  This value should normally be
+	 * set to FALSE.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_CRTSCTS,
 		 g_param_spec_boolean (NM_SETTING_PPP_CRTSCTS,
 						   "CRTSCTS",
-						   "CRTSCTS",
+						   "If TRUE, specify that pppd should set the serial "
+						   "port to use hardware flow control with RTS and CTS "
+						   "signals.  This value should normally be set to "
+						   "FALSE.",
 						   FALSE,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:baud:
+	 *
+	 * If non-zero, instruct pppd to set the serial port to the specified
+	 * baudrate.  This value should normally be left as 0 to automatically
+	 * choose the speed.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_BAUD,
 		 g_param_spec_uint (NM_SETTING_PPP_BAUD,
 						"Baud",
-						"Baud",
+						"If non-zero, instruct pppd to set the serial port to "
+						"the specified baudrate.  This value should normally be "
+						"left as 0 to automatically choose the speed.",
 						0, G_MAXUINT32, 0,
 						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingPPP:mru:
+	 *
+	 * If non-zero, instruct pppd to request that the peer send packets no
+	 * larger than the specified size.  If non-zero, the MRU should be between
+	 * 128 and 16384.
+	 */
 	g_object_class_install_property
 		(object_class, PROP_MRU,
 		 g_param_spec_uint (NM_SETTING_PPP_MRU,
 						"MRU",
-						"MRU",
-						0, G_MAXUINT32, 0,
+						"If non-zero, instruct pppd to request that the peer "
+						"send packets no larger than the specified size.  If "
+						"non-zero, the MRU should be between 128 and 16384.",
+						0, 16384, 0,
 						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingPPP:mtu:
+	 *
+	 * If non-zero, instruct pppd to send packets no larger than the specified
+	 * size.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_MTU,
 		 g_param_spec_uint (NM_SETTING_PPP_MTU,
 						"MTU",
-						"MTU",
+						"If non-zero, instruct pppd to send packets no larger "
+						"than the specified size.",
 						0, G_MAXUINT32, 0,
 						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingPPP:lcp-echo-failure:
+	 *
+	 * If non-zero, instruct pppd to presume the connection to the peer has
+	 * failed if the specified number of LCP echo-requests go unanswered by the
+	 * peer.  The 'lcp-echo-interval' property must also be set to a non-zero
+	 * value if this property is used.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_LCP_ECHO_FAILURE,
 		 g_param_spec_uint (NM_SETTING_PPP_LCP_ECHO_FAILURE,
 						"LCP echo failure",
-						"LCP echo failure",
+						"If non-zero, instruct pppd to presume the connection to "
+						"the peer has failed if the specified number of LCP "
+						"echo-requests go unanswered by the peer.  The "
+						"'" NM_SETTING_PPP_LCP_ECHO_INTERVAL "' property must "
+						"also be set to a non-zero value if this property is "
+						"used.",
 						0, G_MAXUINT32, 0,
 						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingPPP:lcp-echo-interval:
+	 *
+	 * If non-zero, instruct pppd to send an LCP echo-request frame to the peer
+	 * every n seconds (where n is the specified value).  Note that some PPP
+	 * peers will respond to echo requests and some will not, and it is not
+	 * possible to autodetect this.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_LCP_ECHO_INTERVAL,
 		 g_param_spec_uint (NM_SETTING_PPP_LCP_ECHO_INTERVAL,
 						"LCP echo interval",
-						"LCP echo interval",
+						"If non-zero, instruct pppd to send an LCP echo-request "
+						"frame to the peer every n seconds (where n is the "
+						"specified value).  Note that some PPP peers will respond "
+						"to echo requests and some will not, and it is not "
+						"possible to autodetect this.",
 						0, G_MAXUINT32, 0,
 						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 }
