@@ -20,13 +20,14 @@
  */
 
 #ifndef NM_MANAGER_H
-#define NM_MANAGER_H 1
+#define NM_MANAGER_H
 
 #include <glib.h>
 #include <glib-object.h>
 #include <dbus/dbus-glib.h>
 #include "nm-device.h"
 #include "nm-device-interface.h"
+#include "nm-settings.h"
 
 #define NM_TYPE_MANAGER            (nm_manager_get_type ())
 #define NM_MANAGER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_MANAGER, NMManager))
@@ -42,6 +43,8 @@
 #define NM_MANAGER_WIRELESS_HARDWARE_ENABLED "wireless-hardware-enabled"
 #define NM_MANAGER_WWAN_ENABLED "wwan-enabled"
 #define NM_MANAGER_WWAN_HARDWARE_ENABLED "wwan-hardware-enabled"
+#define NM_MANAGER_WIMAX_ENABLED "wimax-enabled"
+#define NM_MANAGER_WIMAX_HARDWARE_ENABLED "wimax-hardware-enabled"
 #define NM_MANAGER_ACTIVE_CONNECTIONS "active-connections"
 
 /* Not exported */
@@ -60,32 +63,21 @@ typedef struct {
 	void (*device_removed) (NMManager *manager, NMDevice *device);
 	void (*state_changed) (NMManager *manager, guint state);
 	void (*properties_changed) (NMManager *manager, GHashTable *properties);
-
-	void (*connections_added) (NMManager *manager, NMConnectionScope scope);
-
-	void (*connection_added) (NMManager *manager,
-				  NMConnection *connection,
-				  NMConnectionScope scope);
-
-	void (*connection_updated) (NMManager *manager,
-				  NMConnection *connection,
-				  NMConnectionScope scope);
-
-	void (*connection_removed) (NMManager *manager,
-				    NMConnection *connection,
-				    NMConnectionScope scope);
 } NMManagerClass;
 
 GType nm_manager_get_type (void);
 
-NMManager *nm_manager_get (const char *config_file,
-                           const char *plugins,
+/* nm_manager_new() should only be used by main.c */
+NMManager *nm_manager_new (NMSettings *settings,
                            const char *state_file,
                            gboolean initial_net_enabled,
                            gboolean initial_wifi_enabled,
                            gboolean initial_wwan_enabled,
+						   gboolean initial_wimax_enabled,
                            GError **error);
 
+NMManager *nm_manager_get (void);
+
 void nm_manager_start (NMManager *manager);
 
 /* Device handling */
@@ -96,7 +88,7 @@ const char * nm_manager_activate_connection (NMManager *manager,
                                              NMConnection *connection,
                                              const char *specific_object,
                                              const char *device_path,
-                                             gboolean user_requested,
+                                             const char *dbus_sender, /* NULL if automatic */
                                              GError **error);
 
 gboolean nm_manager_deactivate_connection (NMManager *manager,
@@ -108,18 +100,6 @@ gboolean nm_manager_deactivate_connection (NMManager *manager,
 
 NMState nm_manager_get_state (NMManager *manager);
 
-/* Connections */
-
-GSList *nm_manager_get_connections    (NMManager *manager,
-                                       NMConnectionScope scope,
-                                       gboolean only_authorized);
-
-gboolean nm_manager_auto_user_connections_allowed (NMManager *manager);
-
-NMConnection * nm_manager_get_connection_by_object_path (NMManager *manager,
-                                                         NMConnectionScope scope,
-                                                         const char *path);
-
 GPtrArray * nm_manager_get_active_connections_by_connection (NMManager *manager,
                                                              NMConnection *connection);
 
